@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
-/* Copyright (C) 2000-2004 Novell, Inc.
+/* Copyright (C) 2001-2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -17,6 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* camel-exchange-folder.c: class for an Exchange folder */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -36,6 +38,7 @@
 #include <camel/camel-mime-filter-crlf.h>
 #include <camel/camel-mime-message.h>
 #include <camel/camel-multipart.h>
+#include <camel/camel-session.h>
 #include <camel/camel-stream-filter.h>
 #include <camel/camel-stream-mem.h>
 
@@ -52,8 +55,8 @@ static void expunge             (CamelFolder *folder,
 static void append_message (CamelFolder *folder, CamelMimeMessage *message,
 			    const CamelMessageInfo *info, char **appended_uid,
 			    CamelException *ex);
-static void set_message_flags (CamelFolder *folder, const char *uid,
-			       guint32 flags, guint32 set);
+static gboolean set_message_flags (CamelFolder *folder, const char *uid,
+				   guint32 flags, guint32 set);
 static void set_message_user_tag (CamelFolder *folder, const char *uid,
 				  const char *name, const char *value);
 static CamelMimeMessage *get_message         (CamelFolder *folder,
@@ -100,15 +103,17 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 }
 
+#define CAMEL_EXCHANGE_SERVER_FLAGS \
+	(CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_ANSWERED_ALL | \
+	 CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_DRAFT | CAMEL_MESSAGE_SEEN)
+
 static void
 init (CamelFolder *folder)
 {
 	folder->folder_flags = CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY | CAMEL_FOLDER_HAS_SEARCH_CAPABILITY;
 	folder->permanent_flags =
-		CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_ANSWERED_ALL |
-		CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_DRAFT |
-		CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN |
-		CAMEL_MESSAGE_USER;
+		CAMEL_EXCHANGE_SERVER_FLAGS | CAMEL_MESSAGE_FLAGGED |
+		CAMEL_MESSAGE_JUNK | CAMEL_MESSAGE_USER;
 }
 
 static void
@@ -280,26 +285,19 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 	camel_object_unref (CAMEL_OBJECT (stream_mem));
 }
 
-static void
+static gboolean
 set_message_flags (CamelFolder *folder, const char *uid,
 		   guint32 flags, guint32 set)
 {
 	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
-	CamelMessageInfo *info;
-	guint32 oldflags, newflags;;
+	gboolean changed;
 
 	if (folder->permanent_flags == 0)
-		return;
+		return FALSE;
 
-	info = camel_folder_summary_uid (folder->summary, uid);
-	g_return_if_fail (info != NULL);
-
-	oldflags = info->flags;
-	CAMEL_FOLDER_CLASS (parent_class)->set_message_flags (folder, uid, flags, set);
-	newflags = info->flags;
-	camel_folder_summary_info_free (folder->summary, info);
-	if (oldflags == newflags)
-		return;
+	changed = CAMEL_FOLDER_CLASS (parent_class)->set_message_flags (folder, uid, flags, set);
+	if (!changed)
+		return FALSE;
 
 	camel_stub_send_oneway (exch->stub, CAMEL_STUB_CMD_SET_MESSAGE_FLAGS,
 				CAMEL_STUB_ARG_FOLDER, folder->full_name,
@@ -307,6 +305,7 @@ set_message_flags (CamelFolder *folder, const char *uid,
 				CAMEL_STUB_ARG_UINT32, set,
 				CAMEL_STUB_ARG_UINT32, flags,
 				CAMEL_STUB_ARG_END);
+	return TRUE;
 }
 
 static void
@@ -336,20 +335,21 @@ fix_broken_multipart_related (CamelMimePart *part)
 	CamelMultipart *multipart, *new;
 	CamelMimePart *subpart;
 	int i, count, broken_parts;
-
-	content_type = camel_mime_part_get_content_type (part);
+	
 	content = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-	if (header_content_type_is (content_type, "message", "rfc822")) {
+	
+	content_type = content->mime_type;
+	if (camel_content_type_is (content_type, "message", "rfc822")) {
 		fix_broken_multipart_related (CAMEL_MIME_PART (content));
 		return;
 	}
 
-	if (!header_content_type_is (content_type, "multipart", "*"))
+	if (!camel_content_type_is (content_type, "multipart", "*"))
 		return;
 	multipart = CAMEL_MULTIPART (content);
 	count = camel_multipart_get_number (multipart);
 
-	if (header_content_type_is (content_type, "multipart", "related") &&
+	if (camel_content_type_is (content_type, "multipart", "related") &&
 	    camel_medium_get_header (CAMEL_MEDIUM (part), "X-MimeOLE"))
 		broken_parts = count - 1;
 	else
@@ -680,14 +680,14 @@ find_parent (CamelExchangeFolder *exch, const char *thread_index)
 	int dlen;
 
 	decoded = g_strdup (thread_index);
-	dlen = base64_decode_simple (decoded, strlen (decoded));
+	dlen = camel_base64_decode_simple (decoded, strlen (decoded));
 	if (dlen < 5) {
 		/* Shouldn't happen */
 		g_free (decoded);
 		return NULL;
 	}
 
-	parent = base64_encode_simple (decoded, dlen - 5);
+	parent = camel_base64_encode_simple (decoded, dlen - 5);
 	g_free (decoded);
 
 	msgid = g_hash_table_lookup (exch->thread_index_to_message_id,
@@ -806,19 +806,23 @@ camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfo *info;
+	CamelFolderChangeInfo *changes;
 
 	info = camel_folder_summary_uid (folder->summary, uid);
 	if (!info)
 		return;
 
-	flags |= (info->flags & (CAMEL_MESSAGE_ATTACHMENTS | CAMEL_MESSAGE_FLAGGED));
+	flags |= (info->flags & ~CAMEL_EXCHANGE_SERVER_FLAGS);
 
 	if (info->flags != flags) {
 		info->flags = flags;
 		camel_folder_summary_touch (folder->summary);
+
+		changes = camel_folder_change_info_new ();
+		camel_folder_change_info_change_uid (changes, uid);
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
-					    "message_changed",
-					    (char *)uid);
+					    "folder_changed", changes);
+		camel_folder_change_info_free (changes);
 	}
 }
 
@@ -830,6 +834,7 @@ camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfo *info;
+	CamelFolderChangeInfo *changes;
 
 	info = camel_folder_summary_uid (folder->summary, uid);
 	if (!info)
@@ -838,13 +843,17 @@ camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
 	camel_tag_set (&info->user_tags, name, value);
 
 	camel_folder_summary_touch (folder->summary);
+	changes = camel_folder_change_info_new ();
+	camel_folder_change_info_change_uid (changes, uid);
 	camel_object_trigger_event (CAMEL_OBJECT (exch),
-				    "message_changed", (char *)uid);
+				    "folder_changed", changes);
+	camel_folder_change_info_free (changes);
 }
 
 gboolean
 camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
-				 const char *name, const char *folder_dir,
+				 const char *name, guint32 camel_flags,
+				 const char *folder_dir,
 				 CamelStub *stub, CamelException *ex)
 {
 	CamelExchangeFolder *exch = (CamelExchangeFolder *)folder;
@@ -862,7 +871,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		short_name = name;
 	camel_folder_construct (folder, parent, name, short_name);
 
-	if (camel_mkdir_hier (folder_dir, S_IRWXU) != 0) {
+	if (camel_mkdir (folder_dir, S_IRWXU) != 0) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not create directory %s: %s"),
 				      folder_dir, g_strerror (errno));
@@ -905,7 +914,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 	}
 
 	if (stub) {
-		gboolean ok;
+		gboolean ok, create = camel_flags & CAMEL_STORE_FOLDER_CREATE;
 
 		exch->stub = stub;
 
@@ -918,13 +927,14 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		for (i = 0; i < summary->len; i++) {
 			info = summary->pdata[i];
 			uids->pdata[i] = (char *)camel_message_info_uid (info);
-			flags->data[i] = info->flags & ~CAMEL_MESSAGE_ATTACHMENTS;
+			flags->data[i] = info->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
 			camel_tag_list_free (&info->user_tags);
 		}
 
 		camel_operation_start (NULL, _("Scanning for changed messages"));
 		ok = camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_FOLDER,
 				      CAMEL_STUB_ARG_FOLDER, name,
+				      CAMEL_STUB_ARG_UINT32, create,
 				      CAMEL_STUB_ARG_STRINGARRAY, uids,
 				      CAMEL_STUB_ARG_BYTEARRAY, flags,
 				      CAMEL_STUB_ARG_RETURN,
@@ -940,6 +950,9 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER)
 			folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
+		if (folder_flags & CAMEL_STUB_FOLDER_FILTER_JUNK)
+			folder->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;
+
 		camel_exchange_summary_set_readonly (folder->summary, folder_flags & CAMEL_STUB_FOLDER_READONLY);
 
 		camel_operation_start (NULL, _("Fetching summary information for new messages"));
