@@ -31,8 +31,10 @@
 #include "e2k-propnames.h"
 #include "e2k-uri.h"
 #include "e2k-utils.h"
+#include "exchange-config-listener.h"
 
 #include <libedataserver/e-xml-hash-utils.h>
+#include <libedataserver/e-source-list.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -333,12 +335,51 @@ create_folder (ExchangeHierarchy *hier, EFolder *parent,
 static ExchangeAccountFolderResult
 remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 {
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
+	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
+	const char *folder_type, *physical_uri;
+
 	if (folder != hier->toplevel)
 		exchange_hierarchy_removed_folder (hier, folder);
 
 	if (folder == hier->toplevel || exchange_hierarchy_is_empty (hier))
 		hierarchy_foreign_cleanup (hier);
 
+	/* Temp Fix for remove fav folders. see #59168 */
+        /* remove ESources */
+        folder_type = e_folder_get_type_string (folder);
+        physical_uri = e_folder_get_physical_uri (folder);
+                                                                                                     
+        if (strcmp (folder_type, "calendar") == 0) {
+                cal_source_list = e_source_list_new_for_gconf (
+                                        gconf_client_get_default (),
+                                        conf_key_cal);
+                remove_esource (hier->account, conf_key_cal, physical_uri,
+                                &cal_source_list, FALSE, FALSE);
+                e_source_list_sync (cal_source_list, NULL);
+                g_object_unref (cal_source_list);
+        }
+        else if (strcmp (folder_type, "tasks") == 0) {
+                task_source_list = e_source_list_new_for_gconf (
+                                        gconf_client_get_default (),
+                                        conf_key_tasks);
+                remove_esource (hier->account, conf_key_tasks, physical_uri,
+                                &task_source_list, FALSE, FALSE);
+                e_source_list_sync (task_source_list, NULL);
+                g_object_unref (task_source_list);
+        }
+        else if (strcmp (folder_type, "contacts") == 0) {
+                cont_source_list = e_source_list_new_for_gconf (
+                                        gconf_client_get_default (),
+                                        conf_key_contacts);
+                remove_esource (hier->account, conf_key_contacts, physical_uri,
+                                        &cont_source_list, FALSE, TRUE);
+                e_source_list_sync (cont_source_list, NULL);
+                g_object_unref (cont_source_list);
+	}
+
 	return EXCHANGE_ACCOUNT_FOLDER_OK;
 }
 
@@ -360,7 +401,9 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *folder)
 static void
 add_href (ExchangeHierarchy *hier, EFolder *folder, gpointer hrefs)
 {
-	g_ptr_array_add (hrefs, (char *)e_folder_exchange_get_internal_uri (folder));
+	char *uri = g_strdup (e_folder_exchange_get_internal_uri (folder));
+
+	g_ptr_array_add (hrefs, (gpointer) uri);
 }
 
 static GPtrArray *
@@ -388,7 +431,73 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 				       const char *folder_name,
 				       EFolder **folder)
 {
-	return create_internal (hier, hier->toplevel, folder_name, NULL, folder);
+	ExchangeAccountFolderResult result;
+	const char *folder_type = NULL;
+	const char *physical_uri = NULL;
+	char *new_folder_name;
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
+	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
+
+	result =  create_internal (hier, hier->toplevel, folder_name, NULL, folder);
+
+	if (result == EXCHANGE_ACCOUNT_FOLDER_OK) {
+		// Add the esources
+		folder_type = e_folder_get_type_string (*folder);
+		physical_uri = e_folder_get_physical_uri (*folder);
+		new_folder_name = g_strdup_printf("%s's %s", 
+					hier->owner_name, folder_name);
+
+		if (!(strcmp (folder_type, "calendar")) ||
+		!(strcmp (folder_type, "calendar/public"))) {
+			cal_source_list = e_source_list_new_for_gconf (
+						gconf_client_get_default (),
+						conf_key_cal);
+			add_esource (hier->account,
+				     conf_key_cal,
+				     new_folder_name,
+				     physical_uri,
+				     &cal_source_list,
+				     FALSE);
+			e_source_list_sync (cal_source_list, NULL);
+			g_object_unref (cal_source_list);
+
+		}
+		else if (!(strcmp (folder_type, "tasks")) ||
+			 !(strcmp (folder_type, "tasks/public"))) {
+				task_source_list = e_source_list_new_for_gconf (
+						gconf_client_get_default (),
+						conf_key_tasks);
+				add_esource (hier->account,
+				     conf_key_tasks,
+				     new_folder_name,
+				     physical_uri,
+				     &task_source_list,
+				     FALSE);
+				e_source_list_sync (task_source_list, NULL);
+				g_object_unref (task_source_list);
+
+		}
+		else if (!(strcmp (folder_type, "contacts")) ||
+			 !(strcmp (folder_type, "contacts/public")) ||
+			 !(strcmp (folder_type, "contacts/ldap"))) {
+				cont_source_list = e_source_list_new_for_gconf (
+						gconf_client_get_default (),
+						conf_key_contacts);
+
+				add_esource (hier->account,
+				     conf_key_contacts,
+				     new_folder_name,
+				     physical_uri,
+				     &cont_source_list,
+				     TRUE);
+				e_source_list_sync (cont_source_list, NULL);
+				g_object_unref (cont_source_list);
+		}
+	}
+
+	return result;
 }
 
 static ExchangeHierarchy *
@@ -403,12 +512,6 @@ hierarchy_foreign_new (ExchangeAccount *account,
 	ExchangeHierarchyForeign *hfor;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
-	g_return_val_if_fail (hierarchy_name != NULL, NULL);
-	g_return_val_if_fail (physical_uri_prefix != NULL, NULL);
-	g_return_val_if_fail (internal_uri_prefix != NULL, NULL);
-	g_return_val_if_fail (owner_name != NULL, NULL);
-	g_return_val_if_fail (owner_email != NULL, NULL);
-	g_return_val_if_fail (source_uri != NULL, NULL);
 
 	hfor = g_object_new (EXCHANGE_TYPE_HIERARCHY_FOREIGN, NULL);
 
@@ -454,6 +557,8 @@ exchange_hierarchy_foreign_new (ExchangeAccount *account,
 	GHashTable *props;
 	xmlDoc *doc;
 
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
+	
 	hier = hierarchy_foreign_new (account, hierarchy_name,
 				      physical_uri_prefix,
 				      internal_uri_prefix,
@@ -498,6 +603,9 @@ exchange_hierarchy_foreign_new_from_dir (ExchangeAccount *account,
 	GHashTable *props;
 	xmlDoc *doc;
 
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
+	g_return_val_if_fail (folder_path != NULL, NULL);
+
 	mf_path = g_build_filename (folder_path, "hierarchy.xml", NULL);
 	doc = xmlParseFile (mf_path);
 	g_free (mf_path);
